#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: DIYer22@github
@mail: ylxx@live.com
Created on Tue Aug  6 20:36:47 2019
"""

from boxx import *
from boxx import pathjoin, tmpboxx, impt

import unittest

with impt(".."):
    from zcs.config import CfgNode

    CN = CfgNode
    from zcs import argument


class TestCfgNode(unittest.TestCase):
    def setUp(self):
        self.cfg = cfg = CN()

        cfg.DATA = CN()
        # cfg.DATA is config for first dataset
        # but also the default base of other cfg.DATA2/3 and cfg.TSET cfg.VAL
        cfg.DATA.DIR = argument(None, str, "Dataset dir")

        cfg.DATA.SIZE = argument(
            512, int, help="The format of dataset directory struct"
        )

        cfg.DATA.NUM_CLASS = argument(
            2, int, "Number of classes to predict (including background)."
        )

        cfg.DATA.IMG_MEAN = (104.00698793, 116.66876762, 122.67891434)

        cfg.DATA2 = cfg.DATA.clone_as_base()

        cfg.DATA2.DIR = "dir2"

        cfg.DATA3 = cfg.DATA.clone_as_base(exclude=["DIR"])

    def testBase(self):
        cfg = self.cfg.clone()
        cfg.merge_from_list_or_str(
            "DATA.DIR dir1 DATA.NUM_CLASS 10 DATA.SIZE 1024 DATA2.SIZE 256"
        )
        cfg.DATA2.update_placeholder_from_base(cfg.DATA)

        self.assertTrue(cfg.DATA2.DIR == "dir2")
        self.assertTrue(cfg.DATA2.IMG_MEAN == cfg.DATA.IMG_MEAN)
        self.assertTrue(cfg.DATA.SIZE == 1024)
        self.assertTrue(cfg.DATA2.SIZE == 256)
        self.assertTrue(cfg.DATA.NUM_CLASS == cfg.DATA2.NUM_CLASS == 10)

        cfg.DATA3.update_placeholder_from_base(cfg.DATA2)
        self.assertTrue(cfg.DATA3.DIR == None)

    def testDump(self):
        import yaml

        cfg = self.cfg.clone()
        yamlp = pathjoin(tmpboxx(), "test.yaml")
        cfg.dump(yamlp)
        cfg_dict = yaml.load(open(yamlp))
        cfgd = CfgNode(cfg_dict)
        self.assertTrue(str(cfg.dump()) == str(cfgd.dump()))

    def tearDown(self):
        pass


if __name__ == "__main__":
    unittest.main()
    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: DIYer22@github
@mail: ylxx@live.com
Created on Fri Jul 19 21:25:18 2019
"""
import sys
import argparse


def str2bool(s):
    if isinstance(s, str):
        if s.lower() in ("yes", "true", "t", "y", "1"):
            return True
        elif s.lower() in ("no", "false", "f", "n", "0"):
            return False
    if s in (None, True, False, 1, 0):
        return bool(s)
    raise argparse.ArgumentTypeError('Boolean value expected. But got "%s"' % s)


def ints(s):
    """Return a list of int in the string
    """
    import re

    return list(map(int, re.findall(r"-?\d+\d*", s)))


def str2index(s):
    """
    return a index or slice
    """
    if ":" in s:  # is slice

        class GetSlice:
            @classmethod
            def __getitem__(self, slicee):
                return slicee

        return eval("GetSlice()[%s]" % s.replace("[", "").replace("]", ""))
    else:
        return ints(s)


def fstring(eval_locals=None):
    """Treate the string as fstring

    Usage:
        >>> agument(type=fstring())
    """
    if eval_locals is None:
        eval_locals = sys._getframe(1).f_locals

    def _fstring(s):
        code = 'f"""%s"""' % s
        return eval(code, eval_locals)

    return _fstring


def try_return_None(type_fun):
    """If a string s == "None", return None
    else return type_fun(s)
    """

    def _type(s):
        if s.strip() in ("None",):
            return None
        else:
            return type_fun(s)

    return _type


str_or_None = try_return_None(str)


if __name__ == "__main__":
    aa = 55
    typee = fstring(dict(aa=3))
    s = typee("aa={aa}")
    assert s == "aa=3", s

    l = list(range(6))

    s = ":2"
    assert l[str2index(s)] == [0, 1]
    s = "[4:]"
    assert l[str2index(s)] == [4, 5]
    s = ":4:2"
    assert l[str2index(s)] == [0, 2]
    s = "4, 2"
    assert str2index(s) == [4, 2]

    pass



__version__ = "0.1.17"
__description__ = "Z Configuration System: a flexible powerful configuration system which takes advantage of both argparse and yacs"
__license__ = "MIT"
__author__ = "DIYer22"
__author_email__ = "ylxx@live.com"
__maintainer__ = "DIYer22"
__maintainer_email__ = "ylxx@live.com"
__github_username__ = "DIYer22"
__url__ = "https://github.com/DIYer22/zcs"
__support__ = "https://github.com/DIYer22/zcs/issues"
__classifiers__ = [
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Intended Audience :: Developers",
    "Intended Audience :: Science/Research",
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: DIYer22@github
@mail: ylxx@live.com
Created on Fri Jul 19 13:30:51 2019
"""
import os
import copy
import yaml
from functools import wraps
import argparse
import yacs.config as yacs


def identity(x):
    return x


_None_ = {None}


class argument(argparse._AttributeHolder):
    """Same API with "parser.add_argument", but remove name and flags, ex. "--opt"
    
    Usage:    
        >>> cfg.FOO = argument(default=None, type=int, help="FOO is a int type, default is None")
    """

    def __init__(
        self,
        default=_None_,
        type=_None_,
        help=_None_,
        metavar=_None_,
        action=_None_,
        nargs=_None_,
        const=_None_,
        choices=_None_,
        required=_None_,
        dest=_None_,
    ):
        if default is None and type is not _None_:
            # when default is None
            # make sure ATTR could set back to None by args.opts
            from .type import try_return_None

            type = try_return_None(type)
        dic = dict(
            default=default,
            type=type,
            help=help,
            metavar=metavar,
            action=action,
            nargs=nargs,
            const=const,
            choices=choices,
            required=required,
            dest=dest,
        )
        dic = dict(filter(lambda x: x[1] is not _None_, dic.items()))
        self.__dict__.update(dic)


class CfgNode(yacs.CfgNode):
    __allow_cover__ = True

    @wraps(yacs.CfgNode.__init__)
    def __init__(self, *l, **kv):
        super(CfgNode, self).__init__(*l, **kv)
        parser = argparse.ArgumentParser(prog="cfg")
        self.__dict__["__parser__"] = parser
        self.__dict__["__action_dic__"] = parser._option_string_actions

    @classmethod
    def _create_config_tree_from_dict(cls, dic, key_list):
        """
        Create a configuration tree using the given dict.
        Any dict-like objects inside dict will be treated as a new CfgNode.

        Args:
            dic (dict):
            key_list (list[str]): a list of names which index this CfgNode from the root.
                Currently only used for logging purposes.
        """
        dic = copy.deepcopy(dic)
        for k, v in dic.items():
            if isinstance(v, dict):
                # Convert dict to CfgNode
                dic[k] = cls(v, key_list=key_list + [k])
            else:
                pass
        return dic

    def _get_parser_actions(self):
        return self.__dict__["__parser__"], self.__dict__["__action_dic__"]

    def __setattr__(self, name, value):
        if self.is_frozen():
            raise AttributeError(
                "Attempted to set {} to {}, but CfgNode is immutable".format(
                    name, value
                )
            )
        yacs._assert_with_logging(
            name not in self.__dict__,
            "Invalid attempt to modify internal CfgNode state: {}".format(name),
        )
        parser, action_dic = self._get_parser_actions()
        argk = name if name.startswith("--") else ("--" + name)
        if isinstance(value, argument):  # if argument
            arg = value
        elif isinstance(value, CfgNode):
            value.__dict__["__parser__"].prog = parser.prog + "." + name
            arg = argument(default=value, type=identity, help="CfgNode")
        else:  # if only value is same as yacs
            arg = argument(
                default=value, help='default is "%s", without argument' % str(value)
            )
        # allow replace in parser
        if self.__allow_cover__ and argk in action_dic:
            action_dic.pop(argk)
        kwargs = {
            # set default type
            "type": identity
            if isinstance(value, str)
            else self._decode_cfg_value
        }
        kwargs.update(arg.__dict__)
        parser.add_argument(argk, **kwargs)
        value = arg.__dict__.get("default", None)
        self[name] = value

    def merge_from_other_cfg(self, cfg_other):
        """Merge `cfg_other` into this CfgNode."""
        _merge_a_into_b(cfg_other, self, self, [])

    @classmethod
    def _load_cfg_py_source(cls, filename):
        """Load a config from a Python source file."""
        module = yacs._load_module_from_file("yacs.config.override", filename)
        yacs._assert_with_logging(
            hasattr(module, "cfg"),
            "Python module from file {} must have 'cfg' attr".format(filename),
        )
        VALID_ATTR_TYPES = {dict, CfgNode, yacs.CfgNode}
        yacs._assert_with_logging(
            type(module.cfg) in VALID_ATTR_TYPES,
            "Imported module 'cfg' attr must be in {} but is {} instead".format(
                VALID_ATTR_TYPES, type(module.cfg)
            ),
        )
        return cls(module.cfg)

    def merge_from_list(self, cfg_list):
        """Merge config (keys, values) in a list (e.g., from command line) into
        this CfgNode. For example, `cfg_list = ['FOO.BAR', 0.5]`.
        """
        yacs._assert_with_logging(
            len(cfg_list) % 2 == 0,
            "Override list has odd length: {}; it must be a list of pairs".format(
                cfg_list
            ),
        )
        root = self
        for full_key, v in zip(cfg_list[0::2], cfg_list[1::2]):
            if root.key_is_deprecated(full_key):
                continue
            if root.key_is_renamed(full_key):
                root.raise_key_rename_error(full_key)
            key_list = full_key.replace(">", ".").split(".")
            d = self
            for subkey in key_list[:-1]:
                yacs._assert_with_logging(
                    subkey in d, "Non-existent key: {}".format(full_key)
                )
                d = d[subkey]
            subkey = key_list[-1]
            yacs._assert_with_logging(
                subkey in d, "Non-existent key: {}".format(full_key)
            )
            value = _parser_action(d, subkey, v)
            d[subkey] = value

    def merge_from_list_or_str(self, cfg_list_str):
        cfg_list = cfg_list_str
        if isinstance(cfg_list_str, str):
            cfg_list = cfg_list_str.strip().split()
        return self.merge_from_list(cfg_list)

    def dump(self, fname=None, **kwargs):
        """Dump to a string."""

        def convert_to_dict(cfg_node, key_list):
            if not isinstance(cfg_node, CfgNode):
                yacs._assert_with_logging(
                    _valid_type(cfg_node),
                    "Key {} with value {} is not a valid type; valid types: {}".format(
                        ".".join(key_list), type(cfg_node), _VALID_TYPES
                    ),
                )
                return cfg_node
            else:
                cfg_dict = dict(cfg_node)
                for k, v in cfg_dict.items():
                    cfg_dict[k] = convert_to_dict(v, key_list + [k])
                return cfg_dict

        self_as_dict = convert_to_dict(self, [])
        yaml_str = yaml.safe_dump(self_as_dict, **kwargs)
        if fname is not None:
            dir_name = os.path.dirname(fname)
            if not os.path.isdir(dir_name):
                os.makedirs(dir_name)
            with open(fname, "w") as f:
                f.write(yaml_str)
        return yaml_str

    __placeholder__ = "__placeholder_for_CfgNode_base__"

    def clone_as_base(self, exclude=None):
        base = self.clone()
        if exclude is None:
            exclude = []
        if isinstance(exclude, str):
            exclude = [exclude]
        exclude = [key.replace(">", ".") for key in exclude]

        def trun_value_to_placeholder_(cfg, key_list=None):
            if key_list is None:
                key_list = []
            for k, v in cfg.items():
                _key_list = key_list + [k]
                key = ".".join(_key_list)
                if key in exclude:
                    continue
                if isinstance(v, yacs.CfgNode):
                    trun_value_to_placeholder_(v, key_list=_key_list)
                else:
                    cfg[k] = self.__placeholder__

        trun_value_to_placeholder_(base)
        return base

    def update_placeholder_from_base(self, base):
        def copy_base_where_v_is_placeholder_(child, base):
            for k, v in child.items():
                if isinstance(v, str) and v == self.__placeholder__:
                    child[k] = base[k]
                if isinstance(v, yacs.CfgNode):
                    copy_base_where_v_is_placeholder_(child[k], base[k])

        copy_base_where_v_is_placeholder_(self, base)
        return self


_VALID_TYPES = copy.deepcopy(yacs._VALID_TYPES)
_VALID_TYPES.add(type(None))


def _valid_type(value, allow_cfg_node=False):
    return (type(value) in _VALID_TYPES) or (
        allow_cfg_node and isinstance(value, CfgNode)
    )


def _parser_action(node, k, v):
    argk = k if k.startswith("--") else ("--" + k)
    parser, action_dic = node._get_parser_actions()
    if argk not in action_dic:
        return v
    action = action_dic[argk]
    v = parser._get_values(action, [v])
    return v


def _merge_a_into_b(a, b, root, key_list):
    """Merge config dictionary a into config dictionary b, clobbering the
    options in b whenever they are also specified in a.
    """
    yacs._assert_with_logging(
        isinstance(a, dict),
        "`a` (cur type {}) must be an instance of {}".format(type(a), dict),
    )
    yacs._assert_with_logging(
        isinstance(b, CfgNode),
        "`b` (cur type {}) must be an instance of {}".format(type(b), CfgNode),
    )

    for k, v_ in a.items():
        full_key = ".".join(key_list + [k])
        # a must specify keys that are in b
        if k not in b:
            if root.key_is_deprecated(full_key):
                continue
            elif root.key_is_renamed(full_key):
                root.raise_key_rename_error(full_key)
            else:
                raise KeyError("Non-existent config key: {}".format(full_key))
        v = copy.deepcopy(v_)
        # Recursively merge dicts
        if isinstance(v, dict):
            if not isinstance(v, yacs.CfgNode):
                v = yacs.CfgNode(v)
            _merge_a_into_b(v, b[k], root, key_list + [k])
            continue
        if isinstance(v, str):
            if v != CfgNode.__placeholder__:
                v = _parser_action(b, k, v)
        b[k] = v


parser = argparse.ArgumentParser(prog="Z Config System Parser")
parser.add_argument("--config", default="", metavar="FILE", help="path to config file")
parser.add_argument(
    "opts",
    help="Modify config options using the command-line",
    default=[],
    nargs=argparse.REMAINDER,
)


def parse_args(parser=parser):
    args = parser.parse_args()
    return args


def merge_by_args(cfg, args=None):
    if args is None:
        args = parse_args()
    if args.config:
        cfg.merge_from_file(args.config)
    cfg.merge_from_list(args.opts)


if __name__ == "__main__":
    pass



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

"""
Z Configuration System: a flexible powerful configuration system 
which takes advantage of both argparse and yacs
"""
from .__info__ import __version__

from .config import CfgNode, argument, identity
from .config import parser, parse_args, merge_by_args

from .type import str2bool, ints, str2index, fstring, try_return_None, str_or_None
